import re
from telegram.ext import Updater
updater = Updater(token='542204492:AAFYvLsJ0br_ACodBUGEL1QsXURELPV-zak')

dispatcher = updater.dispatcher

import logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=logging.INFO)


from telegram.ext import CommandHandler
from telegram.ext import MessageHandler, Filters
from telegram import InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Updater, CommandHandler, CallbackQueryHandler


import telegram

import datetime

from . import HRauth
from . import HRlo
from . import HRget
from . import HRday
from . import utils

date_today = datetime.datetime.today()

hr_auth = HRauth.HRauth()
hr_get = HRget.HRget(hr_auth)

hr = HRlo.HRlo(hr_auth)

json = hr_get.get(year=date_today.year, month=date_today.month, day=date_today.day)
hr_today = HRday.HRday(json)

def get_today():
   json = hr_get.get(year=date_today.year, month=date_today.month, day=date_today.day)
   hr_today = HRday.HRday(json)
   return hr_today

def echo(bot, update):
    bot.send_message(chat_id=update.message.chat_id, text=update.message.text)

def start(bot, update):
    keyboard = [
               [
                 InlineKeyboardButton("time", callback_data='time(bot, update)'),
                 InlineKeyboardButton("exit", callback_data='2'),
               ],
               [
                 InlineKeyboardButton("stamp", callback_data='2')
               ],
               [
                 InlineKeyboardButton("Option 3", callback_data='3')
               ]
               ]

    reply_markup = InlineKeyboardMarkup(keyboard)

    update.message.reply_text('HRlo commands:', reply_markup=reply_markup)


def stop(bot, update):
    bot.send_message(chat_id=update.message.chat_id, text="stop me please")
    updater.stop()

def bot_print(bot, update, msg):
    #msg = re.sub(r"[.\.]+", "\n", msg)
    bot.sendMessage(parse_mode='HTML', chat_id=update.message.chat_id, text='<pre>'+msg+'</pre>')
    #bot.send_message(chat_id=update.message.chat_id, text="`" + msg + "`", parse_mode=telegram.ParseMode.MARKDOWN)

def _message_format(msg):
    return re.sub(r"[.\.]+", "\n", msg)

def button(bot, update):
    query = update.callback_query

    bot.edit_message_text(text="Selected option: {}".format(query.data),
                          chat_id=query.message.chat_id,
                          message_id=query.message.message_id)

def hrlo(bot, update):
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = hr.report_day()
    msg = _message_format(msg)
    bot_print(bot, update, msg)

def day(bot, update):
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = hr.report_day()
    msg = _message_format(msg)
    bot_print(bot, update, msg)

def week(bot, update):
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = hr.report_week()
    msg = _message_format(msg)
    bot_print(bot, update, msg)

def week_month(bot, update):
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = hr.report_month_weeks()
    msg = _message_format(msg)
    bot_print(bot, update, msg)

def month(bot, update):
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = hr.report_month()
    msg = _message_format(msg)
    bot_print(bot, update, msg)

def exit(bot, update):
    hr_today = get_today()
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = "Today estimated Exits\n"
    lunch = True
    least = False
    msg += "Standard time  [{}]\n  Exit  @ {}\n  Remains  {}\n".format(
            hr_today.HR_workday,
            str(hr_today.exit(lunch=lunch, least=least).strftime("%H:%M")),
            str(hr_today.remains(lunch=lunch, least=least)),
            )
    lunch = True
    least = True
    msg += "At Least lunch [{}]\n  Exit  @ {}\n  Remains  {}\n".format(
            hr_today.HR_workday_least_with_lunch,
            str(hr_today.exit(lunch=lunch, least=least).strftime("%H:%M")),
            str(hr_today.remains(lunch=lunch, least=least)),
            )
    lunch = False
    least = True
    msg += "At Least work  [{}]\n  Exit  @ {}\n  Remains  {}\n".format(
            hr_today.HR_workday_least,
            str(hr_today.exit(lunch=lunch, least=least).strftime("%H:%M")),
            str(hr_today.remains(lunch=lunch, least=least)),
            )
    bot_print(bot, update, msg)

def stamp(bot, update):
    hr_today = get_today()
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = "Today Timestamps\n"
    msg += "[{}]\n".format( ", ".join([ i.time().strftime("%H:%M") for i in hr_today.logs()]))
    bot_print(bot, update, msg)

def time(bot, update):
    hr_today = get_today()
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = "Today Times\n"
    msg += "Uptime         {}\n".format(hr_today.uptime())
    msg += "Worked time %  {:.1f}\n".format( 100.0*hr_today.uptime().total_seconds()/hr_today.time_to_work())
    msg += "Time to work   {}\n".format(utils.to_str(hr_today.time_to_work()))
    msg += "Timenet        {}\n".format(utils.to_str(hr_today.timenet()))
    msg += "Lunch time     {}\n".format(utils.to_str(hr_today['time_lunch']))
    msg += "KO time        {}\n".format(utils.to_str(hr_today['time_ko']))
    bot_print(bot, update, msg)
    return msg

def in_presence(bot, update, workers):
    msg = workers
    bot_print(bot, update, msg)

def help(bot, update):
    bot.send_chat_action(chat_id=update.message.chat_id, action=telegram.ChatAction.TYPING)
    msg = """HRlo_bot help
/day
/week
/week_month
/month

/exit
/stamp
/time
    """
    bot_print(bot, update, msg)

def main():
    msg = hr.report_day()
    msg = str(hr_today)
    msg = re.sub(r"[\.]+", "\n", msg)
    print(msg)
    #print(hr.day())
    #print(type(hr.week()))


    echo_handler = MessageHandler(Filters.text, echo)
    dispatcher.add_handler(echo_handler)

    stop_handler = CommandHandler('stop', stop)
    dispatcher.add_handler(stop_handler)

    dispatcher.add_handler( CommandHandler('start', start) )

    dispatcher.add_handler( CommandHandler('day', day) )
    dispatcher.add_handler( CommandHandler('week', week) )
    dispatcher.add_handler( CommandHandler('week_month', week_month) )
    dispatcher.add_handler( CommandHandler('month', month) )

    dispatcher.add_handler( CommandHandler('exit', exit) )
    dispatcher.add_handler( CommandHandler('stamp', stamp) )
    dispatcher.add_handler( CommandHandler('time', time) )

    dispatcher.add_handler( CommandHandler('help', help) )

    #dispatcher.add_handler( CommandHandler('in', in_presence, pass_workers=True) )

    updater.dispatcher.add_handler(CallbackQueryHandler(button))

    updater.start_polling()

    updater.idle()


if __name__ == '__main__':
   main()




#!/usr/bin/env python3
import sys
import datetime
import calendar
import argparse
import json

from collections import OrderedDict

from .logs.dayutils  import dayutils

from . import HRauth

class HRget(object):

    def __init__(self, HRauth, verbose=False, debug=False):

        self.HRauth = HRauth
        self.verbose = verbose

        # set URLs
        self.sheet_url    = 'https://' + self.HRauth.host() + '/HR-WorkFlow/servlet/hfpr_bcapcarte'
        self.post_url     = 'https://' + self.HRauth.host() + '/HR-WorkFlow/servlet/SQLDataProviderServer'
        self.portal_url   = 'https://' + self.HRauth.host() + '/HRPortal/servlet/SQLDataProviderServer'
        self.presence_url = 'https://' + self.HRauth.host() + '/HRPortal/servlet/Report?ReportName=AAA_ElencoPresenti&m_cWv=Rows%3D0%0A0%5Cu0023m_cMode%3Dhyperlink%0A0%5Cu0023outputFormat%3DCSV%0A0%5Cu0023pageFormat%3DA4%0A0%5Cu0023rotation%3DLANDSCAPE%0A0%5Cu0023marginTop%3D7%0A0%5Cu0023marginBottom%3D7%0A0%5Cu0023marginLeft%3D7%0A0%5Cu0023hideOptionPanel%3DT%0A0%5Cu0023showAfterCreate%3DTrue%0A0%5Cu0023mode%3DDOWNLOAD%0A0%5Cu0023ANQUERYFILTER%3D1%0A0%5Cu0023pRAPPORTO%3D%0A0%5Cu0023pFILIALE%3D%0A0%5Cu0023pUFFICIO%3D%0A0%5Cu0023m_cParameterSequence%3Dm_cMode%2CoutputFormat%2CpageFormat%2Crotation%2CmarginTop%2CmarginBottom%2CmarginLeft%2Cmode%2ChideOptionPanel%2CshowAfterCreate%2CANQUERYFILTER%2CpRAPPORTO%2CpFILIALE%2CpUFFICIO%0A'

        self.session = self.HRauth.session()
        if not debug:
            try:
                self.post = self.HRauth.post()
            except:
                raise ConnectionError("[HRget] *** ERROR *** connecting to portal")
                sys.exit(-1)

        self.cookies = self.session.cookies

        if self.verbose > 1:
           print (">>>USERNAME>>>", self.HRauth.username())
           print (">>>IDEMPLOY>>>", self.HRauth.idemploy())
           #print ("[{}]@{}".format(__class__.__name__, sys._getframe().f_code.co_name))
           print (">>>CODE>>>", self.post.status_code)
           print (">>>HISTORY>>>", self.post.history)
           print (">>>HEADERS>>>", self.post.headers)
           print (">>>COOKIES>>>", self.post.cookies)
           print (">>>LOCATION>>>", self.post.headers['location'])
           #print (">>>PAGE>>>", self.post.text)


    def get_range(self, day_range):
       data = []
       # loop over months of day_range
       for y, m in day_range.months():

          # get month data from HR
          full_month = self.get(y, m)

          # get first and last days for month
          first, last = dayutils.month_bounds( datetime.date(y, m, 1) )
          # to avoid future day: if same month of today's month set last day to today
          if dayutils.is_same_month( datetime.datetime(y, m, 1), datetime.datetime.today() ):
             last = datetime.datetime.today().date()
          # list of month days within day_range
          l = [ i.day for i in dayutils.day_range(first, last) if i in day_range ]
          # extract days within day_range from data of all month
          old_data = full_month['Data']
          data += [ old_data[i-1] for i in l ]

          # save fields
          fields = full_month['Fields']

       if data:
         jret = {'Fields' : fields, 'Data' : data}
         return jret


    def _check_data(self, d, year, month, day):
        # check data in month
        if len(d) < 1:
            raise Exception("[HRget] *** ERROR *** Data not found in month {}-{}".format(year, month))
        # check data in day
        if day:
            try:
                d[day-1]
            except:
                raise Exception("[HRget] *** ERROR *** Data not found in day {}-{}-{}".format(year, month, day))

            if day < 1:
                raise Exception("[HRget] *** ERROR *** illegal day {}".format(day))


    def get(self, year  = datetime.datetime.today().year,
                  month = datetime.datetime.today().month,
                  day   = None):

        num_days_in_month = calendar.monthrange(year, month)[1]

        last_day = num_days_in_month
        if dayutils.is_same_month( datetime.datetime(year, month, 1), datetime.datetime.today() ):
           last_day = datetime.datetime.today().day

        if self.verbose > 1:
            print (">>>NUMDAYMONTH>>>", num_days_in_month)
            print (">>>LASTDAY>>>", last_day)

        headers = {
        #           'Pragma': 'no-cache',
        #           'Cache-Control': 'no-cache'
                  }

        params = {
                  'rows' : 300,
                  'startrow' : '0',
                  'count' : 'false',
                  'cmdhash':'89ff07d888efecba391f40eac7d04e9a',
                  'sqlcmd' : 'rows:hfpr_fcartellino3',
                  'IDCOMPANY':'000001',
                  'IDEMPLOY': self.HRauth.idemploy(),
                  'Anno': "{:d}".format(year),
                  'Mese': "{:0>2}".format(month),
                  'Visualiz':'N',
                  'LaFlexi':'N',
                  'LaFlexiProg':'N',
                  'VisDescr':'',
                  'DADRATIM':'',
                  'DATTMI':'',
                  'HHMM':'',
                  'ImportVoci':'S',
                  'p_NOEDIT_1':'',
                  'p_NOEDIT_2':'',
                  'p_NOEDIT_3':'',
                  'p_NOEDIT_4':'',
                  'p_ADMIN':'',
                  'gCarOri':'N',
                  'gVismmAtt':'',
                  'TACART':'N',
                  'vIDPLANTB':'99999',
                  'vIDPRESGRP':'',
                  'NUMERORIGHE':'2',
                  'StringaCol':'01S&&&02S&&&03S&&&04S&&&05S&&&06S&&&07S&&&08S&&&09N&&&10N&&&11N&&&12N&&&13N&&&14N&&&15N&&&16N&&&17N&&&',
                  'SchedeLavoro':'N',
                  'pFILESITO':'',
                  'pFILGIUS':'0',
                  'TOTHORD':'S',
                  'TOTHECC':'',
                  'TOTFLEX':'',
                  'ABILITANOTESPESE':'N',
                  'TIPONOTESPESE':'1',
                  'ABILITATIMESHEET':'N'
                 }

        p = self.session.post(self.sheet_url, cookies=self.cookies)

        p = self.session.post(self.post_url, headers=headers, cookies=self.cookies, params=params)


        d = p.json()['Data'][:last_day]
        f = p.json()['Fields']

        self._check_data(d, year, month, day)

        if day:
            d = d[day-1]
        
        if self.verbose > 1:
            print ('>>>FIELDS>>>', f)
            print ('>>>DATA>>>', d)

            #for i, item in enumerate(f):
            #    print( item, ' = ', d[i])

        json = {'Fields' : f, 'Data' : d}

        return json


    def totalizators(self,
                     year  = datetime.date.today().year,
                     month = datetime.date.today().month):

        date = datetime.date(year, month, 1) + datetime.timedelta(days=calendar.monthrange(year, month)[1])

        headers = {}

        params = {
                  'rows' : 100,
                  'startrow' : '0',
                  'count' : 'false',
                  'cmdhash':'2e2926382311a0b72e72cd1e1cdc4ccc',
                  'sqlcmd' : 'rows:hfpr_fgadgetconta',
                  'pIDEMPLOY': self.HRauth.idemploy(),
                  'pIDCOMPANY':'000001',
                  'pDATA':date,
                  'pADMIN':'',
                 }

        p = self.session.post(self.sheet_url, cookies=self.cookies)

        p = self.session.post(self.post_url, headers=headers, cookies=self.cookies, params=params)

        d = p.json()['Data'][:-1]
        f = p.json()['Fields']

        json = {'Fields' : f, 'Data' : d}

        return json


    def phone(self, names = [], phones = []):

        headers = {}

        params = {
                  'rows'      : '2000',
                  'startrow'  : '0',
                  'count'     : 'true',
                  'cmdhash'   : 'b55cc94f7a3a372690c14949975ac422',
                  'sqlcmd'    : 'q_rubrica',
                  'pANSURNAM' : '',
                 }

        p = self.session.post(self.portal_url, headers=headers, cookies=self.cookies, params=params)

        try:
           p.json()['Data'][:-1]
        except:
           return OrderedDict()

        json  = {'Fields' : p.json()['Fields'], 'Data' : p.json()['Data'][:-1]}

        return json


#{{{ deprecated
    def phone_old(self, names):

        fields_to_return = ['ANSURNAM', 'ANEMAIL', 'ANTELEF', 'ANMOBILTEL']
        data_to_return   = []

        for n in names:

            name = n.upper()

            headers = {}

            params = {
                      'rows'      : '5',
                      'startrow'  : '0',
                      'count'     : 'true',
                      'cmdhash'   : '8a0d2d1c35e16e1e2135a25505b5dd32',
                      'sqlcmd'    : 'q_rubrica',
                      'queryfilter' :"ANSURNAM like '" + name + "%'",
                      'pANSURNAM' : '',
                     }

            p = self.session.post(self.portal_url, headers=headers, cookies=self.cookies, params=params)

            try:
               list_phone = p.json()['Data'][:-1]
            except:
               return OrderedDict()

            fields_ = p.json()['Fields']

            # convert data to ordered dict

            json_ = {}

            for worker in list_phone:
               json = {k: v for k, v in zip(fields_, worker)}
               l = []
               for k in fields_to_return:
                  if json.get(k):
                     l.append(json[k])
               data_to_return.append(l)

        json_['Fields'] = fields_to_return
        json_['Data']   = data_to_return

        return json_
#}}}


    def presence_single(self, name):

        headers = {}
        params = {
                  'rows'      : '15',
                  'startrow'  : '0',
                  'count'     : 'true',
                  'cmdhash'   : '3ff43ef92f5f9df7f54c554adae919a3',
                  'sqlcmd'    : 'core_qzoomgen',
                  'queryfilter' :"DSVALUE like '{}%'".format(name.upper()),
                  'ANTABLEZOOM':"hrdd_employee00",
                  'ANIDFIELD':"IDEMPLOY",
                  'ANDSFIELD':"ANSURNAM",
                  'ANDSFIELD2':"ANNAME",
                  'IDFILTER':"",
                  'ANQUERYFILTER':"1",
                 }

        p = self.session.post(self.portal_url, headers=headers, cookies=self.cookies, params=params)

        try:
            d = p.json()['Data'][:-1]
        except:
            return

        try:
            w_id = d[-1][0]
        except:
            return

        p = self.session.get(
'https://hr.cineca.it/HRPortal/servlet/Report?ReportName=AAA_ElencoPresenti&m_cWv=Rows%3D0%0A0%5Cu0023m_cMode%3Dhyperlink%0A0%5Cu0023outputFormat%3DCSV%0A0%5Cu0023pageFormat%3DA4%0A0%5Cu0023rotation%3DLANDSCAPE%0A0%5Cu0023marginTop%3D7%0A0%5Cu0023marginBottom%3D7%0A0%5Cu0023marginLeft%3D7%0A0%5Cu0023hideOptionPanel%3DT%0A0%5Cu0023showAfterCreate%3DTrue%0A0%5Cu0023mode%3DDOWNLOAD%0A0%5Cu0023ANQUERYFILTER%3D1%0A0%5Cu0023pRAPPORTO%3D{}%0A0%5Cu0023pFILIALE%3D%0A0%5Cu0023pUFFICIO%3D%0A0%5Cu0023m_cParameterSequence%3Dm_cMode%2CoutputFormat%2CpageFormat%2Crotation%2CmarginTop%2CmarginBottom%2CmarginLeft%2Cmode%2ChideOptionPanel%2CshowAfterCreate%2CANQUERYFILTER%2CpRAPPORTO%2CpFILIALE%2CpUFFICIO%0A'.format(w_id)
)

        csv_data = p.text

        return {'csv_data' : csv_data}


    def presence(self):

        p = self.session.get(self.presence_url)

        csv_data = p.text

        return {'csv_data' : csv_data}


    def read(self, f):
        with open(f, 'r') as fp:
            j = json.load(fp)
        return j

    def dump(self, f, j):
        with open(f, 'w') as fp:
           json.dump(j, fp)



def add_parser(parser):

   date_parser = parser.add_argument_group('date options')

   date_parser.add_argument('-d', '--day',
                            default = datetime.datetime.today().day, type=int,
                            help='select day')

   date_parser.add_argument('-m', '--month',
                            default = datetime.datetime.today().month, type=int,
                            help='select month')

   date_parser.add_argument('-y', '--year',
                            default = datetime.datetime.today().year, type=int,
                            help='select year')


def main ():

    parser = argparse.ArgumentParser(prog='HRget',
                                     description='HR manager utility to get json data from HR portal.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    parser.add_argument('-v', '--verbose',
                        action="count", default=0,
                        help="increase verbosity")

    parser.add_argument('-g', '--get',
                        action='store_true',
                        help="get day")

    parser.add_argument('--phone',
                        action='store_true',
                        help="get phone of workers")

    parser.add_argument('--totalizators',
                        action='store_true',
                        help="get totalizators")

    parser.add_argument('--presence',
                        action='store_true',
                        help="get presence of worker")

    parser.add_argument('--presence-single',
                        dest='presence_single',
                        help="get presence of worker")

    parser.add_argument('--dump',
                        dest='file_out',
                        help="dump to file")

    parser.add_argument('--read',
                        dest='file_input',
                        help="read from file")

    HRauth.add_parser(parser)

    args = parser.parse_args()

    djson = None


    auth = HRauth.HRauth(**vars(args))

    hr_get = HRget(auth, verbose=args.verbose)


    if args.file_input:
        djson = hr_get.read(args.file_input)


    if args.get:

        djson = hr_get.get(year=args.year, month=args.month, day=args.day)

        if args.verbose:
            for k, v in zip(djson['Fields'], djson['Data']):
                print(k, " = ", v)


    if args.totalizators:

        djson = hr_get.totalizators(year=args.year, month=args.month)

        if args.verbose:
            for d in djson['Data']:
                for k, v in zip(djson['Fields'], d):
                    print(k, " = ", v)
                print()


    if args.phone:

        djson = hr_get.phone()

        if args.verbose:
            for d in djson['Data']:
                print("---+")
                for k, v in zip(djson['Fields'], d):
                    print(v)
                print("+---")


    if args.presence:

        djson = hr_get.presence()

        if args.verbose:
            print(djson)


    if args.presence_single:

        djson = hr_get.presence_single(args.presence_single)

        if args.verbose:
            print(djson)


    if args.file_out:
        hr_get.dump(f, djson)



if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import datetime
import argparse

from .logs import dayutils

from .logs.daylog import DayLog

from . import HRauth
from . import HRget

from . import utils

class HRday(DayLog):

    re_logs = re.compile('(\d+)-')

    HR_workday                  = datetime.timedelta(hours= 7, minutes=12)
    HR_workday_least            = datetime.timedelta(hours= 5, minutes= 0)
    HR_workday_least_with_lunch = datetime.timedelta(hours= 6, minutes= 0)

    HR_login_last   = datetime.timedelta(hours=10, minutes= 0)

    # lunch timings
    HR_lunch_start = datetime.time(hour=12, minute= 0)
    HR_lunch_end   = datetime.time(hour=15, minute= 0)
    HR_lunch_time  = datetime.timedelta(hours=0, minutes=30)


    sep_descr = '&&&'

    time_hash = {
        'ko'                                 : 'ORE KO', # NB: KO is time for lunch to subtract from uptime.
        'rol'                                : 'ROL',
        'dayoff'                             : 'FERIE',
        'mission'                            : 'MISSIONE',
        'trip'                               : 'TRASFERTA',
        'bankhours'                          : 'BANCA ORE GODUTA',
        'marital leave'                      : 'CONGEDO MATRIMONIALE',
        'father discharge'                   : 'CONGEDO OBBLIGATORIO PADRE',
        'marrowletting'                      : 'DONAZIONE MIDOLLO',
        'bloodletting'                       : 'DONAZIONE SANGUE',
        'disease'                            : 'MALATTIA',
        'disease_company'                    : 'MALATTIA CARICO AZIENDA',
        'baby disease > 3 yrs'               : 'MALATTIA BAMBINO > 3',
        'baby disease <= 3 yrs'              : 'MALATTIA BAMBINO <= 3 ANNI',
        'optional maternity leave'           : 'MATERN./PATERN. FACOLT. (0 - 6 ANNI)',
        'optional maternity leave not paied' : 'MAT./PAT. FACOLT. NON RETR (7 - 12 ANNI)',
        'relative bereavement'               : 'PERMESSO LUTTO AFFINI',
        'bereavement'                        : 'PERMESSO LUTTO E GRAVI MOTIVI',
        'strike'                             : 'SCIOPERO',
        'employees assembly in office'       : 'ASSEMBLEA PERSONALE IN SEDE',
        'employees assembly outside office'  : 'ASSEMBLEA PERSONALE FUORI SEDE',
    }

    sub_times_to_timenet = ['ko', 'bankhours']
    add_times_to_timenet = ['rol', 'dayoff', 'mission', 'trip', 'bankhours',
                            'marital leave', 'father discharge', 'marrowletting', 'bloodletting',
                            'disease', 'baby disease > 3 yrs', 'baby disease <= 3 yrs',
                            'optional maternity leave', 'optional maternity leave not paied',
                            'relative bereavement', 'bereavement', 'strike',
                            'employees assembly in office', 'employees assembly outside office']

    sub_times_to_hr = ['rol', 'dayoff',
                       'marital leave', 'father discharge', 'marrowletting', 'bloodletting',
                       'disease', 'baby disease > 3 yrs', 'baby disease <= 3 yrs',
                       'optional maternity leave', 'optional maternity leave not paied',
                       'relative bereavement', 'bereavement', 'strike']

    time_for_is_working = ['dayoff',
                           'marital leave', 'father discharge', 'marrowletting', 'bloodletting',
                           'disease', 'disease_company', 'baby disease > 3 yrs', 'baby disease <= 3 yrs',
                           'bloodletting', 'marrowletting',
                           'optional maternity leave', 'optional maternity leave not paied',
                           'relative bereavement', 'bereavement', 'strike']

    times_to_add = ['time_lunch', 'time_ko', 'time_2work', 'lunch', 'anomaly', 'mission']

    def __init__(self, json=None, label=None):

        self._now = datetime.datetime.today()
        self['label'] = label

        self._init_times()

        DayLog.__init__(self)

        if not json:
           return

        self.HR = {k: v for k, v in zip(json['Fields'], json['Data'])}

        # get data from HR ...

        # get number of anomalies from HR
        self['anomaly'] = int(self.HR['ANOMALIE'])
        # get working time to do for HR
        self['HR working time'] = self._get_hr_real_work_time()
        # get uptime/nettime for HR
        self['HR times']['up'], self['HR times']['net'] = self._get_hr_times()
        # get lunch benefits
        self['lunch'] = self.lunch()
        # get mission benefits
        self['mission'] = self.mission()
        # get HR times
        for k in self.time_hash.keys():
           self['HR times'][k] = self._get_hr_time(self.time_hash[k])

        # ... end get data from HR


        # init data for Daylog ...

        # get date from HR
        date = datetime.datetime.strptime(self.HR['DATA'], "%Y-%m-%d")
        # get logs from TIMBRATURE of HR
        logs = self.re_logs.findall( self.HR['TIMBRATURE'] )

        # refine HR logs
        logs = [ i[0:2] + ':' + i[2:4] for i in logs ]

        # add now logs if today and odd logs
        if date.date() == self._now.date() and len(logs)%2:
            logs.append( self._now.strftime('%H:%M') )

        # init DayLog class
        DayLog.__init__(self, date, logs)

        # ... end init data for DayLog

        self['time_lunch'] = self._get_lunch_time()
        self['time_ko'] = self._get_ko_time()
        self['time_2work'] = self._get_hr_real_work_time()

        # add timenet key to Daylog class
        self['timenet'] = self._get_timenet()

        # update uptime with mission/business trip times
        for k in ['trip', 'mission', 'bloodletting']:
           self['uptime'] += self['HR times'][k]


# __str__ {{{

    def __str__ (self):
       s = ''
       if not self.date():
           return s
       #s += DayLog.__str__(self) + '\n'

       if self['label']:
          s += "."*25 + "{}\n".format(self['label'])

       # date
       s += "{:.<25}".format( "Date" )
       s += "{}\n".format( str(self.date().date()) )

       # not working day
       if not self.working():
           s += "{:.<25}".format( "Working day" )
           s += "{}\n".format( self.working() )

       # uptime
       s += "{:.<25}".format( "Uptime" )
       s += "{:<10}".format( utils.to_str(self.uptime()) )
       if not self.is_today():
          s += " {} ".format( "for HR" )
          s += "{}".format( utils.to_str(self['HR times']['up']) )
       s += '\n'

       # timenet
       s += "{:.<25}".format( "Timenet" )
       s += "{:<10}".format( utils.to_str(self.timenet()) )
       if not self.is_today():
          s += " {} ".format( "for HR" )
          s += "{}".format( utils.to_str(self['HR times']['net']) )
       s += '\n'

       # time to work
       s += "{:.<25}".format( "Time to work" )
       s += "{:<10}".format( utils.to_str(self.time_to_work()) )
       s += '\n'

       if self.time_to_work():
           # percentage of worked time
           s += "{:.<25}".format( "Worked time in %" )
           s += "{:.1f}%".format( 100.0*self.uptime().total_seconds()/self.time_to_work() )
           s += '\n'

       if not self.is_today():
          s += "{:.<25}".format( "Lunch" )
          s += "{}\n".format( self['lunch'] )
       # lunch time
       s += "{:.<25}".format( "Lunch time" )
       s += "{}".format( utils.to_str(self['time_lunch']) )
       s += '\n'

       # KO time
       if self['HR times']['ko']:
          s += "{:.<25}".format( "KO time" )
          s += "{}\n".format( utils.to_str(self['HR times']['ko']) )
       if self['time_ko'] and not self['HR times']['ko'] and self.working():
          s += "{:.<25}".format( "KO time" )
          s += "{}\n".format( utils.to_str(self['time_ko']) )

       # ROL time
       if self['HR times']['rol']:
          s += "{:.<25}".format( "ROL time" )
          s += "{}\n".format( self['HR times']['rol'] )

       # DAYOFF time
       if self['HR times']['dayoff']:
          s += "{:.<25}".format( "DayOff time" )
          s += "{}\n".format( self['HR times']['rol'] )

       # TRIP time
       if self['HR times']['trip']:
          s += "{:.<25}".format( "Business trip time" )
          s += "{}\n".format( self['HR times']['trip'] )

       # MISSION time
       if self['HR times']['mission']:
          s += "{:.<25}".format( "Mission time" )
          s += "{}\n".format( self['HR times']['mission'] )

       if self.mission():
          s += "{:.<25}".format( "Mission" )
          s += "{}\n".format( self['mission'] )

       if self.logs():
          s += "{:.<25}".format( "TimeStamps" )
          s += "[{}]\n".format( ", ".join([ i.time().strftime("%H:%M") for i in self.logs()]) )

       if self.anomaly():
          s += "{:.<25}".format( "Anomaly" )
          s += "{}\n".format( str(self.anomaly()) )

       #if not self.working():
       #   return s
       if self.is_today():
          s += "{:-<25}\n".format( "---- Estimated Exits " )
          # with lunch
          s += "{:.<25}{}\n".format( "Standard time", utils.to_str(self.HR_workday) )
          s += "{:.<25}".format( "Remains" )
          s += "{}\n".format( utils.to_str(self.remains(lunch=True, least=False)) )
          s += "{:.<25}".format( "Estimated exit" )
          s += "{}\n".format( str(self.exit(lunch=True, least=False).strftime("%H:%M")) )
          # at least with lunch
          s += "{:.<25}{}\n".format( "At least with lunch time", utils.to_str(self.HR_workday_least_with_lunch) )
          s += "{:.<25}".format( "Remains" )
          s += "{}\n".format( utils.to_str(self.remains(lunch=True, least=True)) )
          s += "{:.<25}".format( "Estimated exit" )
          s += "{}\n".format( str(self.exit(lunch=True, least=True).strftime("%H:%M")) )
          # at least
          s += "{:.<25}{}\n".format( "At least working time", utils.to_str(self.HR_workday_least) )
          s += "{:.<25}".format( "Remains" )
          s += "{}\n".format( utils.to_str(self.remains(lunch=False, least=True)) )
          s += "{:.<25}".format( "Estimated exit" )
          s += "{}\n".format( str(self.exit(lunch=False, least=True).strftime("%H:%M")) )
       return s

# }}}

    def __repr__(self):
       return str(self.date().date())


    def __add__(self, other):

       a = HRday()

       sum_daylog = DayLog.__add__(self, other)

       a['logs']   = sum_daylog.logs()
       a['date']   = sum_daylog.date()
       a['uptime'] = sum_daylog.uptime()

       a['timenet'] = self.timenet() + other.timenet()
       a['anomaly'] = self.anomaly() + other.anomaly()

       a['HR working time'] = self['HR working time'] + other['HR working time']
       a['lunch'] = self['lunch'] + other['lunch']
       a['mission'] = self['mission'] + other['mission']

       for k in self['HR times'].keys():
          a['HR times'][k] = self['HR times'][k] + other['HR times'][k]

       for k in self.times_to_add:
           a[k] = self[k] + other[k]

       return a


    def _init_times (self):
        """Inititalize times."""

        self['timenet'] = 0
        self['anomaly'] = 0

        self['HR working time'] = 0
        self['lunch'] = 0
        self['mission'] = 0

        self['HR times'] = {}

        self['HR times']['net'] = 0
        for k in list(self.time_hash.keys()) + ['up']:
           self['HR times'][k] = datetime.timedelta(0)


    def _get_timenet(self):
       """Return timenet in seconds."""

       if self.holiday():
          return 0

       time = self.uptime()

       # times to subtract
       for k in self.sub_times_to_timenet:
          time -= self._get_hr_time(self.time_hash[k])

       if self.get('time_ko') and self.is_today():
           time -= self['time_ko']

       # times to add
       for k in self.add_times_to_timenet:
          time += self._get_hr_time(self.time_hash[k])

       # subtract HR work day
       time -= self.HR_workday

       # convert in seconds
       time_sec = time.total_seconds()

       return time_sec


    def _check_hr_work_time(self):
        """Check HR work time in DESCRORARIO key."""

        if self.HR['DESCRORARIO'].strip() == 'NON IN FORZA':
            raise Exception("[HRday] ***ERROR***\nWorker not employed in date {}".format(self.date().date()))


    def _get_hr_work_time(self):
       """Return working time for HR in seconds.
          Get data from ORARIOTEO key.
          If day is holiday return 0.
       """
       if self.holiday():
          return 0

       self._check_hr_work_time()

       _time_delta = datetime.timedelta(0)
       for i, j in zip(*[iter(self.HR['ORARIOTEO'].split(' - '))]*2):
           _start = datetime.timedelta(hours=int(i[0:2]), minutes=int(i[2:4]))
           _end   = datetime.timedelta(hours=int(j[0:2]), minutes=int(j[2:4]))
           _time_delta = _time_delta + (_end - _start)

       return _time_delta.total_seconds() # convert to seconds


    def _get_hr_real_work_time(self):
       """Return real working time for HR in seconds involving all hashed times (see time_hash list).
          If day is holiday return 0.
       """
       _hr_work_time_sec = self._get_hr_work_time()
       _hr_work_time     = datetime.timedelta( seconds=_hr_work_time_sec )

       # times to subtract
       for k in self.sub_times_to_hr:
          _hr_work_time -= self._get_hr_time(self.time_hash[k])

       _time_sec  = _hr_work_time.total_seconds()

       return _time_sec


    def _get_hr_times(self):
       _oreord = self.HR['OREORD']
       _oreecc = self.HR['OREECC']

       _oreord_sec = utils.hr2seconds(_oreord)
       _oreecc_sec = utils.hr2seconds(_oreecc)
       _oretot_sec = _oreord_sec + _oreecc_sec

       _uptime  = datetime.timedelta( seconds=_oretot_sec )
       _timenet = _oretot_sec - self['HR working time']
       return _uptime, _timenet


    def _get_hr_data_from_description (self, key):
        """Return VALUE of KEY in DESCRIZIONE1 field of HR data."""
        sep = self.sep_descr
        for k, v in zip(self.HR['DESCRIZIONE1'].split(sep), self.HR['QTA1'].split(sep)):
            if k.startswith(key.upper()):
            #if key in k:
               return v


    def _get_hr_time(self, key):
        """Return KEY time in datetime.timedelta.
           Get data from KEY flag in DESCRIZIONE1 field.
           Return time in datetime.timedelta.
        """
        value = self._get_hr_data_from_description(key)
        if value:
            return utils.hr2time(value)
        else:
            return datetime.timedelta(0)


    def _get_lunch_time(self):
        """Return lunch time between lunch start and end times.
           Return time in datetime.timedelta.
        """
        lunch_start = datetime.datetime.combine( self.date(), self.HR_lunch_start )
        lunch_end   = datetime.datetime.combine( self.date(), self.HR_lunch_end )
        working_time_during_lunch_time = self.uptime( lunch_start, lunch_end )
        if not working_time_during_lunch_time:
            return datetime.timedelta(0)
        # last time
        time_final = lunch_end
        if self._now < lunch_end:
            time_final = self._now
        # start time
        time_start = lunch_start
        if self._now < time_start:
            return datetime.timedelta(0)
        # get lunch time
        time_lunch = time_final - time_start - self.uptime( lunch_start, lunch_end )
        return time_lunch


    def _get_ko_time(self):
        """Return ko time.
           Return time in datetime.timedelta.
        """
        _time_ko = self.HR_lunch_time - self['time_lunch']
        if _time_ko < datetime.timedelta(0):
           _time_ko = datetime.timedelta(0)
        return _time_ko


    def _get_lunch_time_remain(self):
        """Return remaining time to reach HR launch time.
           Return time in datetime.timedelta.
        """
        _lunch_time = self._get_lunch_time()
        if _lunch_time > self.HR_lunch_time:
            return datetime.timedelta(0)
        else:
            return self.HR_lunch_time -_lunch_time


    def report(self):
        return str(self)


    def anomaly(self):
        """Return number of anomalies found."""
        return self['anomaly']


    def timenet(self):
        """Return timenet in seconds."""
        return self['timenet']


    def time_to_work(self):
        """Return working time in seconds."""
        return self._get_hr_work_time()


    def remains(self, lunch=True, least=False):
        if not self.is_today():
           return None

        time_to_work = self.HR_workday + self._get_lunch_time_remain()

        if least and not lunch:
            time_to_work = self.HR_workday_least

        if least and lunch:
            time_to_work = self.HR_workday_least_with_lunch

        r = time_to_work - self.uptime()

        if r.total_seconds() < 0:
           return datetime.timedelta(0)
        else:
           return r


    def exit(self, lunch=True, least=False):
        if not self.is_today():
           return None
        return  (self._now + self.remains(lunch, least)).time()


    def holiday(self):
       """Return True if day is an holiday otherwise return False.
          Check if DESCRORARIO key is equal to SABATO, DOMENICA, FESTIVO.
       """
       _desc = self.HR['DESCRORARIO'].split()[0]
       _holiday = ['SABATO', 'DOMENICA', 'FESTIVO']
       #if [s for s in _holiday if _desc in s]:
       if any(_desc in s for s in _holiday):
           return True
       else:
           return False


    def working(self):
       if not self.logs() and not self.mission() and self.holiday():
          return False

       # check for holiday
       if self.holiday():
           return False

       # check for total rol day
       if self.rol_total():
          return False

       # check for times
       for k in self.time_for_is_working:
           if self.get_time(k):
               return False

       return True


    def lunch(self):
       value = self._get_hr_data_from_description('IND. MENSA')
       if value:
           return int(float(value))
       else:
           return 0


    def mission(self):
       value = self._get_hr_data_from_description('IND. DI MISSIONE')
       if value:
           return int(float(value))
       else:
           return 0


    def get_time(self, key):
        return self._get_hr_time(self.time_hash[key])


    def rol_total(self):
        """Check for entire day ROL.
           Return bool.
           If holiday return False.
        """
        if self.holiday():
            return False

        _rol_time_seconds = self.get_time('rol').total_seconds()
        if _rol_time_seconds == self._get_hr_work_time():
           return True
        else:
           return False


    def is_today(self):
       if self.date().date() == self._now.date():
          return True
       else:
          return False




def main():

    parser = argparse.ArgumentParser(prog='HRday',
                                     description='HR day manager',
                                     formatter_class=argparse.RawTextHelpFormatter)

    # add HRget arguments parser
    HRget.add_parser(parser)
    # add HRauth arguments parser
    HRauth.add_parser(parser)

    args = parser.parse_args()


    hr_auth = HRauth.HRauth(**vars(args))
    hr_get = HRget.HRget(hr_auth, verbose=False)
    json = hr_get.get(year=args.year, month=args.month, day=args.day)

    day = HRday(json)

    print(day)


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import sys

NORMAL = "\033[0m"

BLACK = "\033[0;30m"
BLUE = "\033[0;34m"
GREEN = "\033[0;32m"
CYAN = "\033[0;36m"
RED = "\033[0;31m"
PURPLE = "\033[0;35m"
BROWN = "\033[0;33m"
GRAY = "\033[0;37m"
BOLDGRAY = "\033[1;30m"
BOLDBLUE = "\033[1;34m"
BOLDGREEN = "\033[1;32m"
BOLDCYAN = "\033[1;36m"
BOLDRED = "\033[1;31m"
BOLDPURPLE = "\033[1;35m"
BOLDYELLOW = "\033[1;33m"
WHITE = "\033[1;37m"

class color:

   def __init__(self, c):
      self.color = c

   def __call__(self, f):
      def wrapped(*args, **kw):
         if (sys.stdout.isatty()):
            return self.color + f(*args, **kw) + NORMAL
         else:
           return f(*args, **kw)
      return wrapped


if __name__ == '__main__':

   s1 = 'decorating str works only once (in RED!)'

   col = color(BOLDRED)( str )

   print( col(s1) )




#!/usr/bin/env python3
import unittest
import datetime
from daylog import DayLog

class test_day_null(unittest.TestCase):

   day_null = DayLog()

   def test_none(self):
      self.assertFalse(str(self.day_null))


class test_daylog_uptime(unittest.TestCase):

   knownValues = (
      (DayLog(datetime.datetime.today(), ['09:03', '10:13']), datetime.timedelta(0, 4200)),
         )

   def test_uptime(self):
      d1 = DayLog(datetime.datetime.today(), ['09:03', '10:13'])
      self.assertEqual(d1.uptime(), datetime.timedelta(0, 4200))

   def test_known_uptime(self):
      for daylog, uptime in self.knownValues: 
         self.assertEqual( daylog.uptime(), uptime )


if __name__ == '__main__':
   unittest.main()



#!/usr/bin/env python3
import datetime
import calendar

__author__ = 'so07'
__version__ = '0.1.0'

fmt_id       = "%Y%m%d"
fmt_time     = "%H:%M:%S"
fmt_day      = "%d/%m/%Y"
fmt_weekday  = "%a"
fmt_idtot    = fmt_id + fmt_time
fmt_tot      = fmt_day + " " + fmt_time
fmt_human    = fmt_day + " " + fmt_weekday
fmt_humantot = fmt_day + " " + fmt_time + " " + fmt_weekday

fmt_times = ["%H:%M:%S", "%H:%M", "%H.%M.%S", "%H%M%S", "%H.%M"]

days_abbr = list(calendar.day_abbr)
days_long = list(calendar.day_name)

months_abbr = list(calendar.month_abbr)
months_long = list(calendar.month_name)


def date2day(dt, fmt = fmt_tot):
   return dt.strptime( dt.strftime(fmt), fmt ) 


def str2day(day):
   for fmt in [fmt_idtot, fmt_id]:
      try:
         datetime.datetime.strptime(day, fmt)
      except:
         pass
      else:
         return date2day(datetime.datetime.strptime(day, fmt), fmt)
         break

def str2time(t):
   for fmt in fmt_times:
      try:
         datetime.datetime.strptime(t, fmt)
      except:
         pass
      else:
         return date2day(datetime.datetime.strptime(t, fmt), fmt).time()
         break

def day_from_epoch (day):
   epoch = datetime.datetime.utcfromtimestamp(0)
   d = day - epoch
   return d.days

def sec_from_epoch (day):
   epoch = datetime.datetime.utcfromtimestamp(0)
   d = day - epoch
   return d.total_seconds()


class day_range:

   def __init__ (self, start, end, step = datetime.timedelta(1)):
      self.start = start
      self.end   = end
      self.step  = step

   def __len__(self):
      return len([d for d in self])

   def __str__(self):
      #if len(self) == 1:
      #   return str(self.start)
      #else:
         return "start : {}; end : {}".format(self.start, self.end)

   def __iter__(self):
      self.this_day = self.start
      return self

   def __next__(self):

      this_day = self.this_day
      next_day = this_day + self.step

      if this_day > self.end:
         raise StopIteration

      self.this_day = next_day
      return this_day

   #def __contains__(self, day):
   #   l = [ i for i in self if day.date() == i ]
   #   if l:
   #      return True
   #   else:
   #      return False


   def start(self):
      return self.start
   def end(self):
      return self.end
   def step(self):
      return self.step

   def years(self):
      for y in sorted( set([i.year for i in self]) ):
         yield y

   def months(self):
      for m, y in sorted( set([(i.year, i.month) for i in self]) ):
         yield m, y

   def days(self):
      for i in self:
         yield i.year, i.month, i.day


def week_bounds(day):
   if isinstance(day, datetime.date):
       day = datetime.datetime.combine(day, datetime.datetime.min.time())
   start = day.date() - datetime.timedelta(days = day.weekday())
   end = start + datetime.timedelta(days = 6)
   return start, end

def month_bounds(day):
   last_day_month = calendar.monthrange(day.year, day.month)[1]
   start = datetime.date(day.year, day.month, 1)
   end   = datetime.date(day.year, day.month, last_day_month)
   return start, end

def month_weeks_bounds(day):
    w = calendar.Calendar().monthdatescalendar(day.year, day.month)
    return [ (i[0], i[-1]) for i in w ]

def is_same_month(date1, date2):
   if (date1.year == date2.year) and (date1.month == date2.month) :
      return True
   else:
      return False

def sec2str(seconds):
   m, s = divmod(abs(seconds), 60)
   h, m = divmod(abs(m), 60)

   r = ''
   if seconds < 0:
      r += '-'
   r += "%02d:%02d:%02d" % (h, m, s)
   return r




#!/usr/bin/env python3
"""
Script for HR monitoring.
"""
import time
import logging
import argparse
import datetime

from .dump_presents import dump_presents
from .report import report
from .scripts_utils import logit
from .dump_presents import defaults as dump_presents_defaults
from .report import defaults as report_defaults

time_format = '%Y-%m-%d/%H:%M'

defaults = {
    'log_file' : 'monitor.log',
    'csv_prefix'   : dump_presents_defaults['csv_prefix'],
    'csv_dir'      : dump_presents_defaults['csv_dir'],
    'config_file'  : report_defaults['config_file'],
    'verbose'      : 0,
    'monitor_freq' : 30,
    'monitor_start': datetime.datetime.now().strftime(time_format),
    'monitor_end'  : (datetime.datetime.now()+datetime.timedelta(days=1)).strftime(time_format),
}

def monitor_daemon(**kwargs):

    config = {}
    config.update(defaults)
    config.update(kwargs)

    log = logit(**config)

    log.log("----- monitor daemon {} -----".format(datetime.datetime.now()))
    log.log("starting monitor daemon")
    log.log("start @ {}".format(config['monitor_start']))
    log.log("end @ {}".format(config['monitor_end']))
    log.log("frequency in minutes {}".format(config['monitor_freq']))

    while datetime.datetime.now() < datetime.datetime.strptime(config['monitor_end'], time_format):

        if datetime.datetime.now() > datetime.datetime.strptime(config['monitor_start'], time_format) and \
           datetime.datetime.now() < datetime.datetime.strptime(config['monitor_end'], time_format):

            log.log("dumping presents")

            try:
                dump_presents(**config)
            except Exception as e:
                log.log("dump_presents ends with ERROR: " + str(e))

            log.log("getting data")
            log.log("RESULTS @{}".format(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')))

            try:
                report(datefmt='-', **config)
            except Exception as e:
                log.log("report ends with ERROR: " + str(e))

        log.log("waiting next pool @ " + str(datetime.datetime.now()+datetime.timedelta(minutes=config['monitor_freq'])))

        try:
            time.sleep(config['monitor_freq']*60)
        except Exception as e:
            log.log("waiting ends with ERROR: " + str(e))



def add_parser(parser):

    _parser_monitor_daemon = parser.add_argument_group('monitor daemon options')

    _parser_monitor_daemon.add_argument('-v', '--verbose',
                                        action='count',
                                        default=0,
                                        help='increase verbosity')

    _parser_monitor_daemon.add_argument('--csv-prefix',
                                        default=defaults['csv_prefix'],
                                        help='name prefix for csv file. (default %(default)s)')

    _parser_monitor_daemon.add_argument('--csv-dir',
                                        default=defaults['csv_dir'],
                                        help='directory of csv file. (default %(default)s)')

    _parser_monitor_daemon.add_argument('--log-file',
                                        default=defaults['log_file'],
                                        help='log file. (default %(default)s)')

    _parser_monitor_daemon.add_argument('--start',
                                        dest='monitor_start',
                                        default=defaults['monitor_start'],
                                        help='monitor start. time format is YYYY-MM-DD/HH:MM. (default %(default)s)')

    _parser_monitor_daemon.add_argument('--end',
                                        dest='monitor_end',
                                        default=defaults['monitor_end'],
                                        help='monitor end. time format is YYYY-MM-DD/HH:MM. (default %(default)s)')

    _parser_monitor_daemon.add_argument('--freq',
                                        dest='monitor_freq',
                                        default=defaults['monitor_freq'],
                                        type=int,
                                        help='monitor frequencein minutes. (default %(default)s)')



def main():

    parser = argparse.ArgumentParser(prog='HR monitor',
                                     description=__doc__,
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    args = parser.parse_args()

    monitor_daemon(**vars(args))


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import argparse
import datetime

from .logs.dayutils import dayutils, sec2str


def to_str(x):
    """Convert seconds or datetime.timedelta to string in format HH:MM:SS."""
    if isinstance(x, datetime.timedelta):
        return sec2str(x.total_seconds())
    else:
        return sec2str(x)

def hr2seconds(HRtime):
    """Convert time from HR units to seconds.
       Return seconds in float.
    """
    return float(HRtime) *60.0*60.0


def hr2time(HRtime, format=False):
    """Convert time from HR units to timedelta.
       Return datetime.timedelta.
    """
    dt = datetime.timedelta(seconds=hr2seconds(HRtime))

    if format:
        return dayutils.sec2str(dt.total_seconds())
    else:
        return dt
    #return datetime.timedelta(seconds=hr2seconds(HRtime))


def time_sum(l):
    d = datetime.timedelta(0)
    for i in l:
        is_negative = '-' in i
        if is_negative:
            i = i[1:]
        t = dayutils.str2time(i)
        dt = datetime.timedelta(hours=t.hour, minutes=t.minute, seconds=t.second)
        if is_negative:
            d -= dt
        else:
            d += dt
    return d


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


class HashedDict (dict):

    def __keytransform__(self, key):
        return self.key_table[key]

    def __keytransforminverse__(self, key):
        for k, v in self.key_table.items():
           if v == key:
              return k

    def __init__(self, key_table={}, *args, **kwargs):
        self.update(*args, **kwargs)

    def __getitem__(self, key):
        return super(HashedDict,self).__getitem__(self.__keytransform__(key))

    def __str__(self):
        return str( {self.__keytransforminverse__(k): v for k, v in self.items()} )

    def __repr__(self):
        return repr( {self.__keytransforminverse__(k): v for k, v in self.items()} )

    def _get (self, key):
        return super(HashedDict,self).__getitem__(key)
    def _str (self):
        return super(HashedDict,self).__str__()
    def _repr (self):
        return super(HashedDict,self).__repr__()


def refine_string(_str):
    # remove multiple spaces
    _str = re.sub(' +', ' ', _str)
    return _str

class NameParser(argparse.Action):

     def __call__(self, parser, namespace, values, option_string=None):
         values = [ refine_string(i) for i in values ]
         setattr(namespace, self.dest, values)

class TimeParser(argparse.Action):

     def __call__(self, parser, namespace, values, option_string=None):
         values = [ i.strip() for i in values.split(',') ]
         setattr(namespace, self.dest, values)




#!/usr/bin/env python3
import csv
import argparse
import itertools

from .utils import HashedDict

from . import HRauth
from . import HRget
from . import HRpresence


def add_parser(parser):

   _parser = parser.add_argument_group()

   _parser.add_argument('--report',
                        action="store_true",
                        help="report of company")

   _parser.add_argument('--key',
                        dest = 'key',
                        nargs ='+',
                        help='key of csv data to enquire')

   _parser.add_argument('--value',
                        dest = 'value',
                        nargs ='+',
                        help='value of csv data to filter on key results')

   _parser.add_argument('-v', '--verbose',
                        action="count", default=0,
                        help="increase verbosity")

   _parser.add_argument('--list-keys',
                        action = 'store_true',
                        help='list all keys')



def main():

    parser = argparse.ArgumentParser(prog='HRcompany',
                                     description='HR manager utility for company informations.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    HRauth.add_parser(parser)

    args = parser.parse_args()

    if args.list_keys:
       print( "\n".join(sorted(HRpresence.presence.from_key_to_HRkey.keys())) )
       return

    auth = HRauth.HRauth(**vars(args))

    h = HRget.HRget(auth, verbose=False)

    p = HRpresence.HRpresence( h.presence() )

    if args.report:

        workers    = p.get_all('name')
        bosses     = p.get_all('boss')
        department = p.get_all('department')
        office     = p.get_all('office')

        print("Number of workers    =", len(workers))
        if args.verbose > 1:
            print("   >>>", ", ".join(workers))

        print("Number of bosses     =", len(bosses))
        if args.verbose:
            print("   >>>", ", ".join(bosses))

        print("Number of department =", len(department))
        if args.verbose:
            print("   >>>", ", ".join(department))

        print("Number of office     =", len(office))
        if args.verbose:
            print("   >>>", ", ".join(office))


    if args.key or args.value:

           l = p.get_zip(args.key, args.value)

           print("Number of element =", len(l))

           for i in l:
               print(" - ".join(i))


if __name__ == '__main__':
   main()




#!/usr/bin/env python3
from distutils.core import setup

try:
   from distutils.command.build_py import build_py_2to3 as build_py
except ImportError:
   from distutils.command.build_py import build_py

from HRlo import config

setup(

   name=config.name,
   version=config.version,
   description=config.description,
   long_description=config.long_description,
   author=config.author,
   author_email=config.author_email,
   maintainer=config.maintainer,
   url=config.url,
   download_url=config.url,
   requires=['requests'],
   packages=['HRlo', 'HRlo/logs', 'HRlo/logs/dayutils', 'HRlo/scripts'],
   scripts=['bin/accaerralo', 'bin/HRlo',
            'bin/HRday', 'bin/HRget',
            'bin/HRpresence', 'bin/HRcompany',
            'bin/HRtotalizator', 'bin/HRphone',
            'bin/HRutils', 'bin/HRauth', 'bin/HRscripts'],
   cmdclass={'build_py': build_py},

   classifiers=[
       'Development Status :: 3 - Alpha',
       'Programming Language :: Python :: 3',
       'Programming Language :: Python :: 3.2',
       'Programming Language :: Python :: 3.3',
       'Programming Language :: Python :: 3.4',
   ],

)



#!/usr/bin/python
import re
import sys
import subprocess

print "Reading names ...",
sys.stdout.flush()

cmd = 'HRcompany --key office name --value roma'
o, e = subprocess.Popen( cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True ).communicate()

names = re.findall("ROMA - (.*)", o)

print "done"

print "Workers found =", len(names)

print "Search presents ...",
sys.stdout.flush()

cmd = 'HRpresence --in ' + " ".join([ '"%s"' % n for n in names ])
o, e = subprocess.Popen( cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True ).communicate()

presents = re.findall("(.*) : PRESENTE", o)

print "done"

print "Presents found =", len(presents)

print "\nList of presents: "
print "\n".join(presents)



#!/usr/bin/env python3
import datetime
import dayutils

from daylog import DayLog

class DayLogList(list):

    def __init__(self):
       self.daylog = DayLog()

    def __str__(self):

       s = "[LIST] "

       # date
       s += "{:.<20}".format("Date")
       s += "{}\n".format( ", ".join([ i.date(True) for i in self ]))

       # uptime per day
       s += " "*7 + "{:.<20}".format("Up")
       s += "{}\n".format( ", ".join([ dayutils.sec2str(i.uptime().total_seconds()) for i in self ]))

       # logs
       s += " "*7 + "{:.<20}".format("Logs")
       for j in self:
          s += "[{}] ".format(", ".join([ i.time().strftime(dayutils.fmt_time) for i in j.logs() ]))
       s += '\n'

       # uptime
       s += " "*7 +  "{:.<20}{}\n".format( 'Uptime', dayutils.sec2str(self.daylog.uptime().total_seconds()) )

       return s

    def append(self, args):
       list.append(self, args)
       self.daylog['uptime'] += args['uptime']

